@@ -70,6 +70,7 @@ class DateMatcher(object):
             value.extend(val)
             entities.extend(ent)
             return self.output_format_timestamp(value=value, entities=entities, extractor="date_matcher")
+
         elif short_abs:
             val, ent = self.extract_date_abs_number(short_abs, "short")
             value.extend(val)
@@ -336,7 +337,7 @@ class DateMatcher(object):
                     {
                         "start": entities[i]["start"],
                         "end": entities[i]["end"],
-                        "entity": "date",
+                        "entity": "date_time",
                         "value": value[i],
                         "confidence": 1.0,
                         "extractor": extractor
@@ -344,8 +345,282 @@ class DateMatcher(object):
                 )
             return result
 
+    # [21/03/1997 21-03-1997 21_03-1997, ... ]
+    def extract_date_abs_number(self, patterns, cat="long"):
+        value = []
+        entities = []
+        for pattern, span in patterns:
+            tmp = re.split('(,|\s|\.|-|\/|_)', pattern.strip())
+            month = int(tmp[2])
+            day = int(tmp[0])
+            if day > 1000:
+                year = day
+                day = int(tmp[4])
+                flag = self.check_valid_date(day, month)
+                if not flag:
+                    value.append([])
+                else:
+                    wod = self.week_days[datetime.date(
+                        year, month, day).weekday()]
+                    value.append(
+                        [self.normalize_date((wod, day, month, year))])
+                entities.append({
+                    "start": span[0],
+                    "end": span[1]
+                })
+                continue
+            flag = self.check_valid_date(day, month)
+            if not flag:
+                value.append([])
+            else:
+                if cat == "long":
+                    year = str(tmp[4])
+                    if len(year) == 4:
+                        year = int(year)
+                    elif len(year) == 2:
+                        if int(year) > 50:
+                            year = "19"+year
+                        else:
+                            year = "20"+year
+                        year = int(year)
+                    elif len(year) == 1:
+                        year = int(datetime.datetime.now().year)
+                    wod = self.week_days[datetime.date(
+                        year, month, day).weekday()]
+                    value.append(
+                        [self.normalize_date((wod, day, month, year))])
+                else:
+                    year = None
+                    wod = None
+                    value.append(
+                        [self.normalize_date((wod, day, month, year))])
+            entities.append({
+                "start": span[0],
+                "end": span[1]
+            })
+
+        return value, entities
+
+    def normalize_date(self, input_date):  # (WOD, DD, MM, YYYY)
+        WOD = input_date[0]
+        DD = int(input_date[1])
+        MM = int(input_date[2])
+        YYYY = input_date[3]
+        return (WOD, DD, MM, YYYY)
+
+    # Ex: thứ 5 tuần sau, thứ 2 tuần tới
+    def extract_date_rel_clearly(self, val, ent, wod, day, flag):
+        if not wod and not day:
+            if flag:
+                return val, ent
+            else:
+                values = []
+                entities = []
+                for vs, es in zip(val, ent):
+                    v = vs[int(len(vs)/2)]
+                    values.append([v])
+                    entities.append(es)
+                return values, entities
+        entities = []
+        values = []
+
+        # [[("thứ 2", 22, 2, 2222)], [("thứ 2", 22, 2, 2222), ("thứ 3", 22, 3, 2232)]] # [{"start": 10, "end": 20 },  {"start": 20, "end":25}]
+        for v, e in zip(val, ent):
+            if len(v) == 1:
+                entities.append(v)
+                values.append(e)
+                continue
+            if wod:
+                start_span = 100
+                end_span = 0
+                start_span = min([i[1][0] for i in wod])
+                start_span = min(start_span, e['start'])
+                end_span = max([i[1][1] for i in wod])
+                end_span = max(end_span, e['end'])
+                value = []
+                tmp = [i[0] for i in v]
+                tmp_1 = [i[0] for i in wod]
+                for index, i in enumerate(tmp):
+                    if i in tmp_1:
+                        value.extend([v[index]])
+                if not value:
+                    value = v
+                    entities.append(e)
+                else:
+                    entities.append({
+                        "start": start_span,
+                        "end": end_span
+                    })
+                    values.append(value)
+
+            if day:
+                value = []
+                start_span = 100
+                end_span = 0
+                start_span = min([i[1][0] for i in day])
+                start_span = min(start_span, e['start'])
+                end_span = max([i[1][1] for i in day])
+                end_span = max(end_span, e['end'])
+                tmp = [i[1] for i in v]
+                tmp_1 = [i[0] for i in day]
+                for index, i in enumerate(tmp):
+                    if i in tmp_1:
+                        value.extend([v[index]])
+                if not value:
+                    value = v
+                    entities.append(e)
+                else:
+                    entities.append({
+                        "start": start_span,
+                        "end": end_span
+                    })
+                    values.append(value)
+        return values, entities
+
+    def extract_date_abs_string_number(self, wod, day, month, year):
+        tmp = max(len(wod), len(day), len(month), len(year))
+        # print(tmp)
+        start_span = 100
+        end_span = 0
+        wod_tmp = [None] * tmp
+        day_tmp = [None] * tmp
+        month_tmp = [None] * tmp
+        year_tmp = [None] * tmp
+        if wod:
+            wod = wod * tmp
+            wod = wod[:tmp]
+            wod_tmp = [i[0] for i in wod]
+            start_span = min(
+                [j[1][0] if j[1][0] < start_span else start_span for j in wod])
+            end_span = max(
+                [j[1][1] if j[1][1] > end_span else end_span for j in wod])
+        if day:
+            day = day * tmp
+            day = day[:tmp]
+            day_tmp = [i[0] for i in day]
+            start_span = min(
+                [j[1][0] if j[1][0] < start_span else start_span for j in day])
+            end_span = max(
+                [j[1][1] if j[1][1] > end_span else end_span for j in day])
+        if month:
+            month = month * tmp
+            month = month[:tmp]
+            month_tmp = [i[0] for i in month]
+            start_span = min(
+                [j[1][0] if j[1][0] < start_span else start_span for j in month])
+            end_span = max(
+                [j[1][1] if j[1][1] > end_span else end_span for j in month])
+        if year:
+            year = year * tmp
+            year = year[:tmp]
+            year_tmp = [i[0] for i in year]
+            start_span = min(
+                [j[1][0] if j[1][0] < start_span else start_span for j in year])
+            end_span = max(
+                [j[1][1] if j[1][1] > end_span else end_span for j in year])
+        value = []
+        entities = [{
+            "start": start_span,
+            "end": end_span
+        }]
+        value.append([(w, d, m, y) for w, d, m, y in zip(
+            wod_tmp, day_tmp, month_tmp, year_tmp)])
+        return value, entities
+
+    def normalize_date_2(self, wod, day, month, year):
+        if wod:
+            wod_tmp = []
+            for i in wod:
+                value = re.sub('(thứ)|(thu)|(thuws)|(Thứ)', '', i[0])
+                value = value.strip()
+                value = self.dict_normalize.get(value, value)
+                if value != "chủ nhật":
+                    value = 'thứ ' + str(value)
+                wod_tmp.append((value, i[1]))
+            wod = wod_tmp
+        if day:
+            day_tmp = []
+            for i in day:
+                value = re.sub('(ngày)|(ngay)|(ngafy)', '', i[0])
+                value = re.sub('(tháng)|(thang)', '', value)
+                value = value.strip()
+                value = self.dict_normalize.get(value, value)
+                day_tmp.append((value, i[1]))
+            day = day_tmp
+        if month:
+            month_tmp = []
+            for i in month:
+                value = re.sub('(tháng)|(thang)|(thasng)', '', i[0])
+                value = value.strip()
+                value = self.dict_normalize.get(value, value)
+                month_tmp.append((value, i[1]))
+            month = month_tmp
+        if year:
+            year_tmp = []
+            for i in year:
+                value = re.sub('(năm)|(nam)', '', i[0])
+                value = value.strip()
+                value = int(value)
+                year_tmp.append((value, i[1]))
+            year = year_tmp
+        return wod, day, month, year
+
+    def extract_date_rel_with_adj(self, val, ent, top, middle, bot, text):
+        flag = False
+        top = [(x.group(), x.span()) for x in re.finditer(top, text)]
+        middle = [(x.group(), x.span()) for x in re.finditer(middle, text)]
+        bot = [(x.group(), x.span()) for x in re.finditer(bot, text)]
+        if not top and not middle and not bot:
+            return val, ent, flag
+        else:
+            flag = True
+            values = []
+            entities = []
+            for vs, es in zip(val, ent):
+                if top:
+                    for i in top:
+                        v = vs[:round(len(vs)*0.25)]
+                        # print(v)
+                        values.append(v)
+                        start_span = min(i[1][0], es["start"])
+                        end_span = max(i[1][1], es["end"])
+                        entities.append({
+                            "start": start_span,
+                            "end": end_span
+                        })
+                if middle:
+                    for i in middle:
+                        v = vs[round(len(vs)*0.25):round(len(vs)*0.75)]
+                        values.append(v)
+                        start_span = min(i[1][0], es["start"])
+                        end_span = max(i[1][1], es["end"])
+                        entities.append({
+                            "start": start_span,
+                            "end": end_span
+                        })
+                if bot:
+                    for i in bot:
+                        v = vs[round(len(vs)*0.75):]
+                        values.append(v)
+                        start_span = min(i[1][0], es["start"])
+                        end_span = max(i[1][1], es["end"])
+                        entities.append({
+                            "start": start_span,
+                            "end": end_span
+                        })
+            return values, entities, flag
+
+    def check_valid_date(self, day, month):
+        flag = False
+        day_count_for_month = [0, 31, 28, 31,
+                               30, 31, 30, 31, 31, 30, 31, 30, 31]
+        if month > 12 or month <= 0 or int(day) > day_count_for_month[(month)]:
+            return flag
+        else:
+            return True
+
     def output_format_timestamp(self, value, entities, extractor):
-        if not value:
+        if not value or value == [[]]:
             return {
                 "entities": []
             }
@@ -364,261 +639,4 @@ class DateMatcher(object):
                         "extractor": extractor
                     }
                 )
-            return result
-    # [21/03/1997 21-03-1997 21_03-1997, ... ]
-    def extract_date_abs_number(self, patterns, cat="long"):
-        value=[]
-        entities=[]
-        for pattern, span in patterns:
-            tmp=re.split('(,|\s|\.|-|\/|_)', pattern.strip())
-            day=int(tmp[0])
-            month=int(tmp[2])
-            flag=self.check_valid_date(day, month)
-            if not flag:
-                value.append([])
-            else:
-                if cat == "long":
-                    year=str(tmp[4])
-                    if len(year) == 4:
-                        year=int(year)
-                    elif len(year) == 2:
-                        if int(year) > 50:
-                            year="19"+year
-                        else:
-                            year="20"+year
-                        year=int(year)
-                    elif len(year) == 1:
-                        year=int(datetime.datetime.now().year)
-                    wod=self.week_days[datetime.date(
-                        year, month, day).weekday()]
-                    value.append(
-                        [self.normalize_date((wod, day, month, year))])
-                else:
-                    year=None
-                    wod=None
-                    value.append(
-                        [self.normalize_date((wod, day, month, year))])
-            entities.append({
-                "start": span[0],
-                "end": span[1]
-            })
-
-        return value, entities
-
-    def normalize_date(self, input_date):  # (WOD, DD, MM, YYYY)
-        WOD=input_date[0]
-        DD=int(input_date[1])
-        MM=int(input_date[2])
-        YYYY=input_date[3]
-        return (WOD, DD, MM, YYYY)
-
-    # Ex: thứ 5 tuần sau, thứ 2 tuần tới
-    def extract_date_rel_clearly(self, val, ent, wod, day, flag):
-        if not wod and not day:
-            if flag:
-                return val, ent
-            else:
-                values=[]
-                entities=[]
-                for vs, es in zip(val, ent):
-                    v=vs[int(len(vs)/2)]
-                    values.append([v])
-                    entities.append(es)
-                return values, entities
-        entities=[]
-        values=[]
-
-        # [[("thứ 2", 22, 2, 2222)], [("thứ 2", 22, 2, 2222), ("thứ 3", 22, 3, 2232)]] # [{"start": 10, "end": 20 },  {"start": 20, "end":25}]
-        for v, e in zip(val, ent):
-            if len(v) == 1:
-                entities.append(v)
-                values.append(e)
-                continue
-            if wod:
-                start_span=100
-                end_span=0
-                start_span=min([i[1][0] for i in wod])
-                start_span=min(start_span, e['start'])
-                end_span=max([i[1][1] for i in wod])
-                end_span=max(end_span, e['end'])
-                value=[]
-                tmp=[i[0] for i in v]
-                tmp_1=[i[0] for i in wod]
-                for index, i in enumerate(tmp):
-                    if i in tmp_1:
-                        value.extend([v[index]])
-                if not value:
-                    value=v
-                    entities.append(e)
-                else:
-                    entities.append({
-                        "start": start_span,
-                        "end": end_span
-                    })
-                    values.append(value)
-
-            if day:
-                value=[]
-                start_span=100
-                end_span=0
-                start_span=min([i[1][0] for i in day])
-                start_span=min(start_span, e['start'])
-                end_span=max([i[1][1] for i in day])
-                end_span=max(end_span, e['end'])
-                tmp=[i[1] for i in v]
-                tmp_1=[i[0] for i in day]
-                for index, i in enumerate(tmp):
-                    if i in tmp_1:
-                        value.extend([v[index]])
-                if not value:
-                    value=v
-                    entities.append(e)
-                else:
-                    entities.append({
-                        "start": start_span,
-                        "end": end_span
-                    })
-                    values.append(value)
-        return values, entities
-
-    def extract_date_abs_string_number(self, wod, day, month, year):
-        tmp=max(len(wod), len(day), len(month), len(year))
-        # print(tmp)
-        start_span=100
-        end_span=0
-        wod_tmp=[None] * tmp
-        day_tmp=[None] * tmp
-        month_tmp=[None] * tmp
-        year_tmp=[None] * tmp
-        if wod:
-            wod=wod * tmp
-            wod=wod[:tmp]
-            wod_tmp=[i[0] for i in wod]
-            start_span=min(
-                [j[1][0] if j[1][0] < start_span else start_span for j in wod])
-            end_span=max(
-                [j[1][1] if j[1][1] > end_span else end_span for j in wod])
-        if day:
-            day=day * tmp
-            day=day[:tmp]
-            day_tmp=[i[0] for i in day]
-            start_span=min(
-                [j[1][0] if j[1][0] < start_span else start_span for j in day])
-            end_span=max(
-                [j[1][1] if j[1][1] > end_span else end_span for j in day])
-        if month:
-            month=month * tmp
-            month=month[:tmp]
-            month_tmp=[i[0] for i in month]
-            start_span=min(
-                [j[1][0] if j[1][0] < start_span else start_span for j in month])
-            end_span=max(
-                [j[1][1] if j[1][1] > end_span else end_span for j in month])
-        if year:
-            year=year * tmp
-            year=year[:tmp]
-            year_tmp=[i[0] for i in year]
-            start_span=min(
-                [j[1][0] if j[1][0] < start_span else start_span for j in year])
-            end_span=max(
-                [j[1][1] if j[1][1] > end_span else end_span for j in year])
-        value=[]
-        entities=[{
-            "start": start_span,
-            "end": end_span
-        }]
-        value.append([(w, d, m, y) for w, d, m, y in zip(
-            wod_tmp, day_tmp, month_tmp, year_tmp)])
-        return value, entities
-
-    def normalize_date_2(self, wod, day, month, year):
-        if wod:
-            wod_tmp=[]
-            for i in wod:
-                value=re.sub('(thứ)|(thu)|(thuws)|(Thứ)', '', i[0])
-                value=value.strip()
-                value=self.dict_normalize.get(value, value)
-                if value != "chủ nhật":
-                    value='thứ ' + str(value)
-                wod_tmp.append((value, i[1]))
-            wod=wod_tmp
-        if day:
-            day_tmp=[]
-            for i in day:
-                value=re.sub('(ngày)|(ngay)|(ngafy)', '', i[0])
-                value=re.sub('(tháng)|(thang)', '', value)
-                value=value.strip()
-                value=self.dict_normalize.get(value, value)
-                day_tmp.append((value, i[1]))
-            day=day_tmp
-        if month:
-            month_tmp=[]
-            for i in month:
-                value=re.sub('(tháng)|(thang)|(thasng)', '', i[0])
-                value=value.strip()
-                value=self.dict_normalize.get(value, value)
-                month_tmp.append((value, i[1]))
-            month=month_tmp
-        if year:
-            year_tmp=[]
-            for i in year:
-                value=re.sub('(năm)|(nam)', '', i[0])
-                value=value.strip()
-                value=int(value)
-                year_tmp.append((value, i[1]))
-            year=year_tmp
-        return wod, day, month, year
-
-    def extract_date_rel_with_adj(self, val, ent, top, middle, bot, text):
-        flag=False
-        top=[(x.group(), x.span()) for x in re.finditer(top, text)]
-        middle=[(x.group(), x.span()) for x in re.finditer(middle, text)]
-        bot=[(x.group(), x.span()) for x in re.finditer(bot, text)]
-        if not top and not middle and not bot:
-            return val, ent, flag
-        else:
-            flag=True
-            values=[]
-            entities=[]
-            for vs, es in zip(val, ent):
-                if top:
-                    for i in top:
-                        v=vs[:round(len(vs)*0.25)]
-                        # print(v)
-                        values.append(v)
-                        start_span=min(i[1][0], es["start"])
-                        end_span=max(i[1][1], es["end"])
-                        entities.append({
-                            "start": start_span,
-                            "end": end_span
-                        })
-                if middle:
-                    for i in middle:
-                        v=vs[round(len(vs)*0.25):round(len(vs)*0.75)]
-                        values.append(v)
-                        start_span=min(i[1][0], es["start"])
-                        end_span=max(i[1][1], es["end"])
-                        entities.append({
-                            "start": start_span,
-                            "end": end_span
-                        })
-                if bot:
-                    for i in bot:
-                        v=vs[round(len(vs)*0.75):]
-                        values.append(v)
-                        start_span=min(i[1][0], es["start"])
-                        end_span=max(i[1][1], es["end"])
-                        entities.append({
-                            "start": start_span,
-                            "end": end_span
-                        })
-            return values, entities, flag
-
-    def check_valid_date(self, day, month):
-        flag=False
-        day_count_for_month=[0, 31, 28, 31,
-                               30, 31, 30, 31, 31, 30, 31, 30, 31]
-        if month > 12 or month <= 0 or int(day) > day_count_for_month[(month)]:
-            return flag
-        else:
-            return True
+            return result
\ No newline at end of file
